@@ -8,18 +8,11 @@ use Text::Starfish;
 use File::Copy;
 use Carp;
 use Cwd;
+
 my $comment_width = 14;
 sub cmt_print { my $c=shift; while (length($c)<$comment_width) {
   $c.='.'; } print "$c: "; }
 
-&cmt_print("use test(1)"); ok(1);
-
-# Slash hack was added because the Windows terminal evaluates does not
-# evaluate $vars (the use a %varname% form instead) and therefore would
-# not require the extra escape character at the beginning of the variable.
-my $slash_hack;
-if ($^O =~ m/MSWin/) { $slash_hack = ''; }
-else                 {$slash_hack = '\\'; }
 &my_mkdir('tmp', 'tmp/Text');
 copy('Starfish.pm','tmp/Text/Starfish.pm');
 {
@@ -29,6 +22,15 @@ copy('Starfish.pm','tmp/Text/Starfish.pm');
 }
 chdir 'tmp' or die;
 
+&cmt_print("01-use test"); ok(1);
+
+# Slash hack was added because the Windows terminal evaluates does not
+# evaluate $vars (the use a %varname% form instead) and therefore would
+# not require the extra escape character at the beginning of the variable.
+my $slash_hack;
+if ($^O =~ m/MSWin/) { $slash_hack = '';   }
+else                 { $slash_hack = '\\'; }
+
 &testcase('02-simple_java'); # a simple Java example
 &testcase('03-simple_java'); # a simple Java example, related to previous
 &testcase('04-simple_java', 'replace'); # a simple Java example, replace mode
@@ -36,35 +38,80 @@ chdir 'tmp' or die;
 &testcase('06-addHook', 'replace'); # 01->06-addHook
 
 if (&is_module_available('CGI')) {
-  &testcase('02', 'replace'); # "ok 3" requires CGI module
+  &testcase('07-html-cgi', 'replace'); # requires CGI module
 } else {
-  ok(1); print "\tskipped (ok 3) - CGI module not available\n"; }
+  &cmt_print('07-html-cgi'); ok(1);
+  print "\t07-html-cgi skipped - CGI module not available\n"; }
+
+&testcase('08-tex', 'replace');
+&testcase('09-text', 'replace');
+&testcase('10-tex');
+&testcase('11-make');
+&testcase('12-make');
+&testcase('13-html', 'replace');
+&testcase('14-text');
+&testcase('15-text');
+&testcase('16-rmHook');
+&testcase('17-A_java');
+&testcase('18-p_t_java', 'out');
+&testcase('19-p_t_java', 'out', '-e=$Release=1');
+&testcase('20-simple_html');
+
+##&testcase('21-make');
+$testnum = '21-make';
+&cmt_print($testnum);
+my $testdir = "test-$testnum";
+#!!!if (-d $testdir) { &rm_dir_recursively($testdir) }
+&my_mkdir($testdir);
+chdir $testdir or die;
+my $testfilesdir = "../../testfiles/$testnum";
+my $insource = "$testfilesdir/Makefile.in"; my $inorig = "Makefile.in";
+my $procfile = "Makefile";
+my $outsource = "$testfilesdir/Makefile.out";
+my $outExpected = "Makefile.out-expected";
+mycopy($insource, $inorig);
+mycopy($insource, $procfile);
+mycopy($outsource, $outExpected);
+for (qw(A B C)) { mycopy("$testfilesdir/$_.java", "$_.java"); }
+my @sfishArgs = ( $procfile );
+my $outNew = $procfile;
+putfile('test-description', '# CWD: '.getcwd()."\n".
+	"# Test preparation:\n".
+	"# cp $insource $inorig\n".
+	"# cp $insource $procfile\n".
+	"# cp $outsource $outExpected\n".
+	"# cp $testfilesdir/A.java A.java\n".
+	"# cp $testfilesdir/B.java B.java\n".
+	"# cp $testfilesdir/C.java C.java\n".
+	"# #option: perl -I. -- starfish @sfishArgs\n".
+	"# starfish_cmd( @sfishArgs );\n".
+	"# diff $outExpected $outNew\n".
+	"# If test needs to be updated, input:\n".
+	"# cp $inorig $insource\n".
+	"# Output:\n".
+	"# cp $outNew $outsource\n");
+starfish_cmd( @sfishArgs );
+comparefiles($outExpected, $outNew);
+chdir '..' or die;
 
-&testcase('03', 'replace'); # "ok 4"
-&testcase('05', 'replace'); # "ok 5"
-&testcase('06'); # was 5 "ok 6"
-&testcase('07'); # was 25, Makefile "ok 7"
-&testcase('08'); # Makefile "ok 8"
-&testcase('09', 'replace'); # old 7, html "ok 9"
-&testcase(2);
-&testcase(3);
-&testcase(4);
 &testcase(6, 'out');
 &testcase(8);
 &testcase(9, 'out');
 &testcase(30); # html.sfish to html (16 ok)
-# 10 (ok 17)
+
+&cmt_print('(ok 24)');
 copy('../testfiles/9_java.out', '9_java.out');
 starfish_cmd(qw(-o=10_java.out -e=$Starfish::HideMacros=1 9_java.out));
 ok(getfile('10_java.out'),
    getfile("../testfiles/10_java.out"));
-# 11 (ok 18)
+
+&cmt_print('(ok 25)'); # Macros testing
 copy('../testfiles/10_java.out', '10.java');
 starfish_cmd(qw(-o=11_java.out 10.java));
 ok(getfile('11_java.out'),
    getfile("../testfiles/11_java.out"));
 
-# 12 (ok 19)
+&cmt_print('(ok 26)'); # option mode testing
 `echo "OSNAME | $OSNAME |"`;
 # Skip if it is windows
 if ($^O =~ m/MSWin/) {
@@ -82,44 +129,38 @@ else {
   ok($tmp, "444");
 }
 
-# (ok 20)
-&testcase(13, 'out');
+&testcase(13, 'out'); # ok 24 # macros
 
-# 14 (ok 21)
+# 14
+&cmt_print('(ok 28)');
 copy('../testfiles/13_java.in','14.java');
 #`perl -I. -- starfish -o=14.out -e="$slash_hack\$Star::HideMacros=1" 14.java`;
 starfish_cmd(qw(-o=14.out -e=$Star::HideMacros=1 14.java));
 comparefiles('../testfiles/14.out', '14.out');
 
-    # 15,16
-    copy('../testfiles/15.java','tmp.java');
-    `$^X -I. -- starfish -o=tmp.ERR -e="$slash_hack\$Star::HideMacros=1" tmp.java>tmp1 2>&1`;
-    ok($? != 0);
-    okfiles('../testfiles/15.out', 'tmp1');
+# 15,16
+&cmt_print('(ok 29)');
+copy('../testfiles/15.java','tmp.java');
+`$^X -I. -- starfish -o=tmp.ERR -e="$slash_hack\$Star::HideMacros=1" tmp.java>tmp1 2>&1`;
+ok($? != 0);
+&cmt_print('(ok 30)');
+okfiles('../testfiles/15.out', 'tmp1');
 
-    # 17, old 16
-    copy('../testfiles/16develop.SLeP','tmp.SLeP');
-    copy('../testfiles/16.tex','tmp.tex');
-    `$^X -I. -- starfish tmp.SLeP tmp.tex`;
-	if ($^O =~ m/MSWin/) {
-	    `copy /B /Y tmp.SLeP+tmp.tex tmp1`;
-	}
-	else {
-	    `cat tmp.SLeP tmp.tex>tmp1`;
-	}
-    okfiles('../testfiles/16.out', 'tmp1');
+# 17, old 16 # multiple files
+&cmt_print('(ok 31)');
+copy('../testfiles/16develop.SLeP','tmp.SLeP');
+copy('../testfiles/16.tex','tmp.tex');
+`$^X -I. -- starfish tmp.SLeP tmp.tex`;
+if ($^O =~ m/MSWin/) {
+  `copy /B /Y tmp.SLeP+tmp.tex tmp1`;
+}
+else {
+  `cat tmp.SLeP tmp.tex>tmp1`;
+}
+okfiles('../testfiles/16.out', 'tmp1');
 
-    # 18, old 17
-    copy('../testfiles/p_t.java','tmp.java');
-    `$^X -I. -- starfish -o=tmp1 tmp.java`;
-    okfiles('../testfiles/17.out', 'tmp1');
-
-    # 19, old 18
-    copy('../testfiles/p_t.java', 'tmp.java');
-    `$^X -I. -- starfish -e="$slash_hack\$Release=1" -o=tmp1 tmp.java`;
-    okfiles('../testfiles/18.out', 'tmp1');
-
-# 20, old 19 (ok 27)
+# 20, old 19
+&cmt_print('(ok 32)');
 if ($^O =~ m/MSWin/) {
   skip('Skipped under windows...');
 } else {
@@ -132,7 +173,8 @@ if ($^O =~ m/MSWin/) {
   ok($tmp, "644");
 }
 
-# 21, old 20 has to be done after previous (ok 28)
+# 21, old 20 has to be done after previous
+&cmt_print('(ok 33)');
 if ($^O =~ m/MSWin/) {
   skip('Skipped under windows...');
 } else {
@@ -144,21 +186,21 @@ if ($^O =~ m/MSWin/) {
   ok($tmp, "644");
 }
 
-    # 22, old 21
-    copy('../testfiles/21.html','tmp2.html');
-    `$^X -I. -- starfish -replace -o=tmp1 tmp2.html`;
-    okfiles('../testfiles/21.out', 'tmp1');
+# 22, old 21
+&cmt_print('(ok 34)');
+copy('../testfiles/21.html','tmp2.html');
+`$^X -I. -- starfish -replace -o=tmp1 tmp2.html`;
+okfiles('../testfiles/21.out', 'tmp1');
 
-    # 23
-    &testcase(22);    
+&testcase(22);    
+&testcase(24);
 
-    # 24
-    &testcase(24);
-    #copy('../testfiles/24.py','24.py');
-    #`$^X -I. -- starfish 24.py`;
-    #okfiles('../testfiles/24.py.out', '24.py');
+#copy('../testfiles/24.py','24.py');
+#`$^X -I. -- starfish 24.py`;
+#okfiles('../testfiles/24.py.out', '24.py');
 
 # 26
+&cmt_print('(ok 37)');
 copy('../testfiles/26_include_example.html','26_include_example.html');
 copy('../testfiles/26_include_example1.html','26_include_example1.html');
 starfish_cmd(qw(-replace -o=26-out.html 26_include_example.html));
@@ -176,7 +218,7 @@ sub okfiles {
 	if (!-f $f1)
 	{ die "file $f1 does not exist (to be compared to tmp/$f2)"	}
 	if (! ok(getfile($f2), getfile($f1)) )
-	{ print STDERR "cwd=".getcwd()."Files: $f1 and $f2\n" }
+	{ print TESTLOG "cwd=".getcwd()."Files: $f1 and $f2\n" }
 
     }
 }
@@ -185,26 +227,46 @@ sub okfiles {
 # $infile   - name of the original input test file (in testdir)
 # $procfile - name of the input file when starfish is run on it
 # $outfile  - name of the expected outfile in the testdir
+# @args     - additional sfish arguments
 sub testcase {
   my $testnum = shift; &cmt_print($testnum);
-  my ($infile, $procfile, $replace, $out, $outfile);
+  my ($infile, $procfile, $replace, $out, $outfile, @args);
   my $testdir = "test-$testnum";
-  if (-d $testdir) { &rm_dir_recursively($testdir) }
+  #!!!if (-d $testdir) { &rm_dir_recursively($testdir) }
   &my_mkdir($testdir);
   chdir $testdir or die;
   my $testfilesdir = '../../testfiles';
 
-    # example: &testcase(34, 'in:33_tex.in->34.tex -replace -o=34-slides.tex');
-    if ($#_==0 && $_[0] =~ /^in:(\S*)->(\S*) -replace -o=(\S*)$/) {
-      $infile = $1; $procfile = $2; $outfile = $replace = $3;
-    }
-    elsif ( -e "$testfilesdir/$testnum.in" and $#_==-1) {
-      $infile   = "$testnum.in";
-      $procfile = "$testnum.in";
-      $outfile  = "$testnum.out";
-      if ($testnum =~ /_java$/) { $procfile = "$`.java" }
-    }
-    elsif ( -e "$testfilesdir/$testnum.in" and
+  # example: &testcase(34, 'in:33_tex.in->34.tex -replace -o=34-slides.tex');
+  if ($#_==0 && $_[0] =~ /^in:(\S*)->(\S*) -replace -o=(\S*)$/) {
+    $infile = $1; $procfile = $2; $outfile = $replace = $3;
+  }
+  elsif ( -e "$testfilesdir/$testnum.in" and $#_==-1) {
+    $infile   = "$testnum.in";
+    $procfile = "$testnum.in";
+    $outfile  = "$testnum.out";
+    if ($testnum =~ /_(java|html)$/) {
+      my $ext = $1; $procfile = "$`.$ext"; }
+  }
+  elsif ( -e "$testfilesdir/$testnum.in" and $#_==0 and $_[0] eq 'out') {
+    $infile   = "$testnum.in";
+    $procfile = "$testnum.in";
+    $outfile  = "$testnum.out";
+    $out = $outfile;
+    if ($testnum =~ /^\d+-(\w.*)_java$/) {
+      $procfile = "$1.java"; $out = "$1_out.java" }
+  }
+  elsif ( -e "$testfilesdir/$testnum.in" and $#_==1 and $_[0] eq 'out'
+	  and $_[1]=~ /^-e=/) {
+    $infile   = "$testnum.in";
+    $procfile = "$testnum.in";
+    $outfile  = "$testnum.out";
+    $out = $outfile;
+    push @args, $_[1];
+    if ($testnum =~ /^\d+-(\w.*)_java$/) {
+      $procfile = "$1.java"; $out = "$1_out.java" }
+  }
+  elsif ( -e "$testfilesdir/$testnum.in" and
 	    $#_==0 and $_[0] eq 'out' ) {
 	$infile   = "$testnum.in";
 	$procfile = "$testnum.in";
@@ -263,17 +325,18 @@ sub testcase {
 	$procfile = "$testnum.py";
 	$outfile = "${testnum}.py.out";
     }
-    else { die }
+    else { die "Test files not found" }
 
-    mycopy("$testfilesdir/$infile", $infile);
-    mycopy("$testfilesdir/$infile", $procfile) if $infile ne $procfile;
-    my $outExpected = "$outfile-expected";
-    my $outfile_masked = $outfile;
-    $outfile_masked =~ s/\.tex$/_tex.out/;
-    if (!-f "$testfilesdir/$outfile" && -f "$testfilesdir/$outfile_masked")
-    { mycopy("$testfilesdir/$outfile_masked", "$outfile-expected"); }
-    else
-    { mycopy("$testfilesdir/$outfile", "$outfile-expected"); }
+  mycopy("$testfilesdir/$infile", "$infile-orig");
+  mycopy("$testfilesdir/$infile", $infile);
+  if ($infile ne $procfile) { mycopy("$testfilesdir/$infile", $procfile); }
+  my $outExpected = "$outfile-expected";
+  my $outfile_masked = $outfile;
+  $outfile_masked =~ s/\.tex$/_tex.out/;
+  if (!-f "$testfilesdir/$outfile" && -f "$testfilesdir/$outfile_masked")
+  { mycopy("$testfilesdir/$outfile_masked", "$outfile-expected"); }
+  else
+  { mycopy("$testfilesdir/$outfile", "$outfile-expected"); }
     my $outNew      = $procfile;
     my @sfishArgs = ( '-e=$ver="testver"', $procfile );
     if ($replace) {
@@ -281,15 +344,22 @@ sub testcase {
       $outNew = $replace;
     }
     elsif ($out) {
-      @sfishArgs = ('-e=$ver="testver"', "-o=$out", $procfile);
+      @sfishArgs = ('-e=$ver="testver"', "-o=$out", @args, $procfile);
       $outNew = $out;
     }
 
-    putfile('test-description', '# CWD: '.getcwd().
+    putfile('test-description', '# CWD: '.getcwd()."\n".
+	    "# Test preparation:\n".
+	    "# cp $testfilesdir/$infile $infile-orig\n".
+	    "# cp $testfilesdir/$infile $procfile\n".
 	    "# cp $testfilesdir/$outfile $outfile-expected\n".
-	    "# perl -I. -- starfish @sfishArgs\n".
+	    "# #option: perl -I. -- starfish @sfishArgs\n".
 	    "# starfish_cmd( @sfishArgs );\n".
-	    "# diff $outExpected $outNew\n");
+	    "# diff $outExpected $outNew\n".
+	    "# If test needs to be updated, input:\n".
+	    "# cp $infile-orig $testfilesdir/$infile\n".
+	    "# Output:\n".
+	    "# cp $outNew $testfilesdir/$outfile\n");
     starfish_cmd( @sfishArgs );
     comparefiles($outExpected, $outNew);
     
@@ -316,7 +386,8 @@ sub comparefiles {
       if ($l1 ne $l2) { print STDERR "$f1:$l1", "$f2:$l2" }
       $fc1=$fc1r; $fc2=$fc2r;
     }
-    ok(0);
+    print STDERR "Test failed.\n";
+    ok(0); # File comparison failed
     return;
   }
   ok(1);
